using System.Text;
using DataAcquisitionSolution.PLCAccessors;
using MessagePack;
using Microsoft.Extensions.Hosting;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using ServiceCenter;
using ServiceCenter.Extensions;
using ServiceCenter.Logs;
using ServiceCenter.Redis;
using ServiceCenter.SqlSugars;
using SqlSugar;
using WCS.Core;

namespace DataAcquisitionSolution;

/// <summary>
///     
/// </summary>
public class Worker : BackgroundService
{
    public static readonly string WcsDlog = "WCSDlog";
    public static readonly string Wcsdb = "WCSDB";

    /// <summary>
    ///     ¼
    /// </summary>
    private readonly ILogger<Worker> _logger;

    /// <summary>
    ///     캯
    /// </summary>
    /// <param name="logger">¼</param>
    public Worker(ILogger<Worker> logger)
    {
        _logger = logger;
    }

    /// <summary>
    ///     ִ
    /// </summary>
    /// <param name="stoppingToken">ֹͣ</param>
    /// <returns></returns>
    protected override Task ExecuteAsync(CancellationToken stoppingToken)
    {
        if (stoppingToken.IsCancellationRequested)
            return Task.CompletedTask;

        #region ʼRedis

        var redisConnectionStrings = RedisHub.Default.Check("RedisConnectionStrings") ??
                                     throw new Exception("RedisRedisConnectionStrings");
        var configs = JsonConvert.DeserializeObject<List<DataBaseConnectionString>>(redisConnectionStrings);
        if (configs != null)
            if (configs.All(v => v.Key != "Monitor"))
                throw new Exception("RedisConnectionStringsüRedisDBַ");

        foreach (var redisConnection in configs!)
        {
            RedisHub.CreateContext(redisConnection.ConnectionString, redisConnection.Key);
            switch (redisConnection.Key)
            {
                case "Monitor":
                    RedisHub.SetMonitorContextType(redisConnection.Key);
                    RedisHub.Monitor.Serialize = obj =>
                    {
                        var bytes = MessagePackSerializer.Serialize(obj);
                        return bytes;
                    };
                    RedisHub.Monitor.DeserializeRaw = (bytes, type) =>
                    {
                        var obj = MessagePackSerializer.Deserialize(type, bytes);
                        return obj;
                    };
                    break;

                case "DebugRedisUrl":
                    RedisHub.SetDebugContextType(redisConnection.Key);
                    Configs.DebugRedisUrl = redisConnection.ConnectionString;
                    break;

                case "WMS":
                    RedisHub.SetWMSContextType(redisConnection.Key);
                    break;
            }
        }

        #endregion ʼRedis

        #region ־

        //var logConfigText = RedisHub.Default.Check("LogConfigText") ?? throw new Exception("Redislog4net");
        //var logConfig = JsonConvert.DeserializeObject<LogConfig>(logConfigText);
        //LogHub.SetConfigInfo(logConfig!);

        #endregion ־

        _logger.LogInformation("WCSʼ");
        Encoding.RegisterProvider(CodePagesEncodingProvider.Instance);
        Configs.StringEncoding = Encoding.UTF8;
        //var warehouseName = RedisHub.Default.Check("WarehouseName") ?? throw new Exception("Redisòֿ");
        //if (string.IsNullOrEmpty(warehouseName)) throw new Exception("Redisòֿ");
        //ServiceHub.SetWarehouseName(warehouseName);

        #region ʼݿ

        var dbConnectionStrings =
            RedisHub.Default.Check("DbConnectionStrings") ?? throw new Exception("Redisݿ");
        ServiceHub.DbConnectionStrings =
            JsonConvert.DeserializeObject<List<DataBaseConnectionString>>(dbConnectionStrings);
        if (ServiceHub.DbConnectionStrings != null)
        {
            if (ServiceHub.DbConnectionStrings.All(v => v.Key != Wcsdb))
                throw new Exception("DbConnectionStringsWCSݿַ");
            if (ServiceHub.DbConnectionStrings.All(v => v.Key == Wcsdb && !v.IsDefault))
                throw new Exception("DbConnectionStringsWCSݿΪĬݿ");
            // if (ServiceHub.DbConnectionStrings.All(v => v.Key != WcsDlog)) throw new Exception("DbConnectionStringsWCS־ݿַ");
        }

        //Ϣ
        var connectionConfigs = ServiceHub.DbConnectionStrings!.Select(connectionString => new ConnectionConfig
            {
                ConfigId = connectionString.Key,
                ConnectionString = connectionString.ConnectionString, //ӷִ
                DbType = connectionString.DbType, //ݿ
                IsAutoCloseConnection = true, //trueҪֶclose
                LanguageType = LanguageType.Chinese,
                MoreSettings = new ConnMoreSettings { IsNoReadXmlDescription = true }
            })
            .ToList();
        ;
        SqlSugarHelper.SetDb(new SqlSugarScope(connectionConfigs));

        ServiceHub.DbConnectionStrings.InitDb();

        #endregion ʼݿ

        #region ʼ豸Ϣ

        WorkStart.InitializeDeviceInfo();

        #endregion ʼ豸Ϣ

        //PLC
        Configs.PLCAccessorCreater = new PlcAccessorsCreater();

        BusinessCenter.DataCollectionLoop();

        //#region ʼPLCPLCȡЭ
        //try
        //{
        //    #region е

        //    World.StartAll();

        //    #endregion е

        //    _logger.LogInformation("WCSɹ");
        //}
        //catch (Exception ex)
        //{
        //    _logger.LogError("WCSʧ{0}", ex.Message);
        //}

        //#endregion ʼPLCPLCȡЭ

        LogHub.init();
        return Task.CompletedTask;
    }
}